/*********************************************************************
* Software License Agreement (BSD License)
*
*  Copyright (c) 2010, Rice University
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*   * Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*   * Redistributions in binary form must reproduce the above
*     copyright notice, this list of conditions and the following
*     disclaimer in the documentation and/or other materials provided
*     with the distribution.
*   * Neither the name of the Rice University nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
*********************************************************************/

/* Author: Mark Moll */

#include "ompl/extensions/vortex/VortexStatePropagator.h"
#include "ompl/extensions/vortex/VortexStateSpace.h"
#include "ompl/extensions/vortex/VortexControlSpace.h"

ompl::control::VortexStatePropagator::VortexStatePropagator(const SpaceInformationPtr &si) : StatePropagator(si)
{
    if (VortexStateSpace *oss = dynamic_cast<VortexStateSpace*>(si->getStateSpace().get()))
        env_ = oss->getEnvironment();
    else
        throw Exception("Vortex State Space needed for VortexStatePropagator");
}

void ompl::control::VortexStatePropagator::propagate(const base::State *state,
    const Control* control, const double duration, base::State *result) const
{
    env_->mutex_.lock();

    Vx::VxReal3 f;
    int nsteps = ceil(duration/env_->stepSize_);

    if (!env_->collisionSubscriber_)
        env_->addDefaultCollisionSubscriber();

    // place the Vortex world at the start state
    si_->getStateSpace()->as<VortexStateSpace>()->writeState(state);
    env_->world_->setTimeStep(duration/(double)nsteps);
    env_->invalidCollisionOccured_ = false;
    for (int i=0; i<nsteps; ++i)
    {
        // apply the controls
        env_->applyControl(control->as<RealVectorControlSpace::ControlType>()->values);
        env_->world_->step();
        if (env_->invalidCollisionOccured_)
        {
            result->as<VortexStateSpace::StateType>()->collision_ &= (1 << VortexStateSpace::STATE_COLLISION_VALUE_BIT);
            break;
        }
    }
    // read the final state from the Vortex world
    si_->getStateSpace()->as<VortexStateSpace>()->readState(result);

    env_->mutex_.unlock();
    // update the collision flag for the result state
    result->as<VortexStateSpace::StateType>()->collision_ &= (1 << VortexStateSpace::STATE_COLLISION_KNOWN_BIT);
}

bool ompl::control::VortexStatePropagator::canPropagateBackward(void) const
{
    return false;
}

